﻿using System.ComponentModel;

namespace TestProject
{
    /// <summary>
    /// 테스트 항목
    /// </summary>
    public class TestItem : INotifyPropertyChanged
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 속성 변경시 이벤트 - PropertyChanged

        /// <summary>
        /// 속성 변경시 이벤트
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 명칭
        /// </summary>
        private string name;

        /// <summary>
        /// 선택 여부
        /// </summary>
        private bool isSelected;

        /// <summary>
        /// 확장 여부
        /// </summary>
        private bool isExpanded;

        /// <summary>
        /// 자식 리스트
        /// </summary>
        private BindingList<TestItem> childList;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 명칭 - Name

        /// <summary>
        /// 명칭
        /// </summary>
        public string Name
        {
            get
            {
                return this.name;
            }
            set
            {
                this.name = value;
                
                FirePropertyChangedEvent("Name");
            }
        }

        #endregion
        #region 선택 여부 - IsSelected

        /// <summary>
        /// 선택 여부
        /// </summary>
        public bool IsSelected
        {
            get
            {
                return this.isSelected;
            }
            set
            {
                this.isSelected = value;
                
                FirePropertyChangedEvent("IsSelected");
            }
        }

        #endregion
        #region 확장 여부 - IsExpanded

        /// <summary>
        /// 확장 여부
        /// </summary>
        public bool IsExpanded
        {
            get
            {
                return this.isExpanded;
            }
            set
            {
                this.isExpanded = value;
                
                FirePropertyChangedEvent("IsExpanded");
            }
        }

        #endregion
        #region 자식 리스트 - ChildList

        /// <summary>
        /// 자식 리스트
        /// </summary>
        public BindingList<TestItem> ChildList
        {
            get
            {
                return this.childList;
            }
            set
            {
                this.childList = value;

                FirePropertyChangedEvent("ChildList");
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// private

        #region 속성 변경시 이벤트 발생시키기 - FirePropertyChangedEvent(propertyName)

        /// <summary>
        /// 속성 변경시 이벤트 발생시키기
        /// </summary>
        /// <param name="propertyName">속성명</param>
        private void FirePropertyChangedEvent(string propertyName)
        {
            if(PropertyChanged != null)
            {
                PropertyChangedEventArgs e = new PropertyChangedEventArgs(propertyName);

                PropertyChanged(this, e);
            }
        }

        #endregion
    }
}